#include <xercesc/util/PlatformUtils.hpp>

#include <qapplication.h>
#include "ruledebuggerImpl.h"

void startupXML() {
     try {
	  XMLPlatformUtils::Initialize();
     }
     catch (const XMLException& toCatch) {
	  char* message = XMLString::transcode(toCatch.getMessage());
	  std::cout << "Error during initialization! :\n"
		    << message << "\n";
	  delete [] message;
	  std::exit(1);
     }
}

void shutdownXML() {
     XMLPlatformUtils::Terminate();
}

void printUsage(const char *name) {
     std::cerr << "Usage:  " << name << " [ options... ]\n"
	       << "Options:\n"
	       << "   -f<filename>\tSpecifies which settings file to use\n"
	       << "   -h\t\tThis help text\n\n";
     std::exit(0);
}

int main( int argc, char* argv[] )
{
     startupXML();

     std::string settingsfile = "settings.xml";
     for (int i = 1; argv[i]; ++i) {
	  char *arg = argv[i];
	  if (*arg == '-') {
	       switch (*++arg) {
	       case 'f': 
		    if (*++arg == '\0') printUsage(argv[0]); 
		    settingsfile = arg; break;
	       case 'h':
	       default: printUsage(argv[0]); break;
	       }
	  }
     }

     QApplication app( argc, argv );
     
     RuleDebuggerImpl rdb(settingsfile);
     app.setMainWidget(&rdb);
     rdb.show();
     
     int ret = app.exec();
     shutdownXML();
     return ret;
}

