/******************************************************************************

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

******************************************************************************/

#ifndef DAMERAU_H
#define DAMERAU_H

#include <string>
#include <ctime>
#include <iostream>
#include <fstream>
#include <limits>
#include <list>

#include <boost/random.hpp>
#include <boost/tokenizer.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/tuple/tuple.hpp>

#include "wordlist.hpp"
#include "settings.hpp"
#include "matrix.hpp"
#include "errorwriter.hpp"
#include "wordtag.hpp"
#include "exceptions.hpp"

/**
 * Tha Damerau class introduces spelling errors on word level.
 * There are four error types: insertion, deletion, substitution and transposition
 */
class Damerau {
public:

     Damerau(Wordlist &, Settings &, ErrorWriter &);
     ~Damerau();
     bool apply(std::list<WordTag>&) const;

private:
    typedef boost::mt19937 base_generator_type;
    typedef boost::uniform_int<> distribution_type;
    typedef boost::variate_generator<base_generator_type, distribution_type> generator_type;

     typedef boost::tuple<WordTag, std::string, int> WtStrInt;

     void setProbabilities(const char, const char, const char, const char);
     WtStrInt apply(const WordTag &) const;
     char confuseLetter(const char) const;
     void readConfusionMatrix(const std::string&);

     Wordlist &wordlist;
     Settings &settings;
     ErrorWriter &errorwriter;
     std::string letters;
     Matrix<int> confusionMatrix;
     std::vector<int> confusionSum;
    base_generator_type generator;
    /*distribution_type dst;*/
    mutable generator_type random_number;
     generator_type *rnd;
     int sum;
     int subst;
     int ins;
     int del;
     int transp;
     int numLetters;
     bool allowExistingWords;
     bool forceAllowWords;
     bool allowTagChange;
     bool forceAllowTag;
     std::string reportDamerau;
};

#endif
