/******************************************************************************

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

******************************************************************************/

#ifndef INPUTREADER_H
#define INPUTREADER_H

#include <fstream>
#include <string>
#include <list>
#include <cstdlib>

#include <boost/regex.hpp>
#include <boost/format.hpp>

#include "wordtag.hpp"
#include "wordlist.hpp"
#include "settings.hpp"
#include "exceptions.hpp"

/**
 * The class used to read the input from disk.
 */
class InputReader {
public:
     InputReader(Wordlist &, Settings &);
     ~InputReader();
     void next();
     void printBuffer() const;
     std::list<WordTag>& getBuffer();
     void changeReadOffset(int);
private:
     WordTag readRow();
     Wordlist &wordlist;
     Settings &settings;
     std::ifstream infile;
     std::ofstream outfile;
     std::string format;
     unsigned int bufferSize;
     unsigned int row_read;
     unsigned int row_orig;
     std::string noError;
     std::string existingWord;
     std::string nonExistingWord;
     std::string wordChange;
     std::string noWordChange;
     std::string tagChange;
     std::string noTagChange;
     boost::regex exp;
     std::list<WordTag> buffer;
};

#endif
