/******************************************************************************

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

******************************************************************************/

#ifndef SOUNDERROR_H
#define SOUNDERROR_H

#include <iostream>
#include <string>
#include <list>
#include <map>
#include <fstream>
#include <ctime>

#include <boost/random.hpp>
#include <boost/regex.hpp>
#include <boost/lexical_cast.hpp>

#include "wordlist.hpp"
#include "settings.hpp"
#include "errorwriter.hpp"
#include "wordtag.hpp"
#include "exceptions.hpp"

/**
 * A class with a stupid name.
 * This class can be used to create many types of simple spelling
 * errors within a word. For example to convert "skj" to "sk"
 * with a certain probability if an "u" follows (skjuta becomes skuta).
 */
class SoundError {
public:

     SoundError(Wordlist &, Settings &, ErrorWriter &);     
     bool apply(std::list<WordTag>&) const;

protected:
    typedef boost::mt19937 base_generator_type;
    typedef boost::uniform_real<> distribution_type;
    typedef boost::variate_generator<base_generator_type, distribution_type> random_real;
    typedef std::list<std::pair<std::string,double> > ListPStrDbl;
    typedef std::map<boost::regex,std::pair<double,ListPStrDbl> > Datamap;

     std::string replaceMatchedExpressions(const std::string&, const boost::match_results<std::string::const_iterator> &) const;
     void insert(const std::string &, const std::string &, double);
     void readFromFile(const std::string&);

     Wordlist &wordlist;
     Settings &settings;
     ErrorWriter &errorwriter;
     const char escapeChar;
     mutable boost::mt19937 generator;
     Datamap data;
     bool allowExistingWords;
     bool forceAllowWords;
     bool allowTagChange;
     bool forceAllowTag;
     std::string reportSoundError;
};

#endif
